# concept based on JF.draw_sine_wave by Jim Folts, with help from Jim Hamilton and TIG...
module JcB
  # the logic is only loaded if the menu ruby is activated...
  module SineCircle
    extend self

    unless defined? @radius
      @radius      = 0.5.m
      @segments    = 96
      @modulo      = 6
      @scale       = 4
      @keep_circle = 'Yes'
      @cylinder    = 'Yes'
      @explode     = 'Yes'
    end

    def user_input
      prompts = [
        TRANSL8['Radius'] + ':',
        TRANSL8['Segments'] + ':',
        TRANSL8['Waves'] + ':',
        TRANSL8['Wave Height'] + ':',
        TRANSL8['keep Circle'] + ':',
        TRANSL8['keep Cylinder'] + ':',
        TRANSL8['Explode All'] + ':']

      defaults = [@radius, @segments, @modulo, @scale, @keep_circle, @cylinder, @explode]

      list = ['', '', '', '1|2|3|4|5|6|7|8|9', 'No|Yes', 'No|Yes', 'No|Yes']

      input = UI.inputbox(prompts, defaults, list, TRANSL8['Sine Circle Details'])
      return unless input

      @radius      = input[0].to_l
      @segments    = input[1].to_i
      @modulo      = input[2].to_i
      @scale       = input[3].to_i
      @keep_circle = input[4]
      @cylinder    = input[5]
      @explode     = input[6]

      input_error = @segments.modulo(@modulo) != 0
      if input_error
        UI.messagebox( TRANSL8['Use Number of Circle Segments that are divisible by the Number of Wave Segments'])
        p 'retry'
        @modulo -= 1 unless 1
        user_input
      else
        single_wave
      end
    end

    def single_wave
      model = Sketchup.active_model
      entities = model.active_entities

      # wrap geometry creation in an undo...
      Sketchup.active_model.start_operation('Sine Circle')
      circle_grp = entities.add_group
      circle_gents = circle_grp.entities
      # create the base circle perpendicular to the normal or Z axis
      edges = circle_gents.add_circle(ORIGIN, Z_AXIS, @radius, @segments)


      # p     edges.first.length.to_mm
      # collect the verts
      verts_ary = []
      edges.first.curve.vertices.each { |vert| verts_ary << vert.position.to_a }

      edges.each(&:find_faces) rescue edges.each{|i| i.find_faces}
      face = circle_gents.grep(Sketchup::Face)[0]
      face.reverse!

      face.pushpull(@radius) if @cylinder == 'Yes'

      unless @keep_circle == 'Yes' || @cylinder == 'Yes'
        entities.erase_entities(circle_grp)
      end
      # p     verts_ary.length

      # add the new vert positions
      i = 0
      new_pts = []
      segs = @segments / 2
      verts_ary.each do |vert_pos|
        i += 1
        ang = X_AXIS.angle_between(vert_pos)
        height = @scale * Math.sin(@modulo * ang)
        if i <= segs # of base circle
          new_pts << (vert_pos[0..1] << height)
        else # flip
          new_pts << (vert_pos[0..1] << -height)
        end
      end

      # make the curve
      grp = Sketchup.active_model.active_entities.add_group
      gents = grp.entities
      gents.add_curve(new_pts)

      if @cylinder == 'Yes'
        point = Geom::Point3d.new(0, 0, @radius / 2)
        t = Geom::Transformation.new point
        grp.move! t
      end

      if @explode == 'Yes'
        grp.explode
        circle_grp.explode unless circle_grp.deleted?
      end

      Sketchup.active_model.commit_operation
    end # def single_wave
  end # module SineCircle
end # module JcB

